//Name: Justin Fiedler
//Project: HW5
//Course: CS 540

import java.io.*;
import java.util.*;
import java.util.Random;


public class HW5
{	private ArrayList points = new ArrayList();
	private Guassian[] g;
	public static boolean debug;

	public static void main(String args[])
	{	boolean debug = false;

		String	datafile	= args[0];
		int k 				= Integer.parseInt(args[1]);
		int	seed 			= Integer.parseInt(args[2]);


		if(args.length == 4  && args[3].equals("-d"))
		{    debug = true;
		}


		//java HW5 <datafile.txt> <k> <randomSeed> [-d]
		new HW5(datafile, k, seed, debug);
	}

	public HW5(String datafile, int k, int seed, boolean debug)
	{	this.debug = debug;

		//create guassians
		g = new Guassian[k];

		//initialize gaussians
		Random r = new Random(seed);
		for(int i = 0; i < g.length; i++)
		{	g[i] = new Guassian();
			g[i].ux = r.nextDouble() * 300.0 + 200.0;
			g[i].ox = r.nextDouble() * 75.0;
			g[i].uy = r.nextDouble() * 300.0 + 200.0;
			g[i].oy = r.nextDouble() * 75.0;
		}

		//read in the data from the file
		readPoints(datafile);

		//run EM algorithm
		EM();

	}

    /* readPoints
	 * accepts the name of a file from which to read a list of points
     * using MyFileReader and stores them in 'points'
     */
	public void readPoints(String fileName)
	{	try
		{	MyFileReader fr = new MyFileReader(fileName, " ");
			String line = fr.readLine();
			while(line != null)
			{	double x = Double.parseDouble(fr.nextToken());
				double y = Double.parseDouble(fr.nextToken());

				points.add(new MyPoint(x,y));

				line = fr.readLine();
			}
		}
		catch(FileNotFoundException e)
		{	System.out.println(fileName + " not found");
			System.exit(0);
		}
		catch(IOException e)
		{	System.out.println("Error reading from " + fileName);
			System.exit(0);
		}
	}

	/* EM
	 * runs the EM algorithm on the given guassians and examples
	 */
	public void EM()
	{	while(!converged())
		{	// E Step
			if(debug) System.out.println("Probablities");

			//find the most probable cluster for each data point
			for(int i = 0; i < points.size(); i++)
            {	int best_cluster = -1;
				double best_prob = -1;

				MyPoint p = (MyPoint)points.get(i);

            	if(debug) System.out.print(p + ":    ");

            	for(int j = 0; j < g.length; j++)
            	{	double curr_prob = g[j].probability(p);

            		if(debug) System.out.print(curr_prob + " ");

            		if(curr_prob > best_prob)
            		{	best_cluster = j;
            			best_prob = curr_prob;
            		}
            	}

            	if(debug) System.out.println();

            	p.set_cluster(best_cluster);
            }

            // M Step
            if(debug)
            {	System.out.println("Clusters");
            	for(int i = 0; i < g.length; i++)
            	{	System.out.print(g[i] + ":   ");
            		ArrayList pts = get_guassian_pts(i);
            		for(int j = 0; j < pts.size(); j++)
            		{	System.out.print((MyPoint)pts.get(j) + " ");
            		}
            		System.out.println();
            	}
            	System.out.println();
            }
            set_u();
            set_o();
        }

        System.out.println("CONVERGED");
        for(int i = 0; i < g.length; i++)
        {	System.out.println(g[i]);
        }

	}

	/* set_u
	 * change the current model(mu values) for each guassian based on the
	 * data points that associated themselves with it
	 */
	public void set_u()
	{	MyPoint p;
		for(int i = 0; i < g.length; i++)
		{	//get points that belong to the current guassian
			ArrayList curr_guassian_pts = get_guassian_pts(i);

            //u = (1/n) * sum 1 to n {vi}
            double sum_x = 0;
            double sum_y = 0;
            for(int j = 0; j < curr_guassian_pts.size(); j++)
            {	p = (MyPoint)curr_guassian_pts.get(j);
            	sum_x += p.x;
            	sum_y += p.y;
        	}
        	g[i].ux	= sum_x / curr_guassian_pts.size();
        	g[i].uy = sum_y / curr_guassian_pts.size();
		}
	}

	/* set_o
	 * change the current model(sigmoid values) for each guassian based on the
	 * data points that associated themselves with it
	 */
	public void	set_o()
	{	MyPoint p;
		for(int i = 0; i < g.length; i++)
		{	//get points that belong to the current guassian
			ArrayList curr_guassian_pts = get_guassian_pts(i);

            //o = Math.sqrt((1/n) * sum 1 to n {(vi - u)^2}
            double sum_x = 0;
            double sum_y = 0;
            for(int j = 0; j < curr_guassian_pts.size(); j++)
            {	p = (MyPoint)curr_guassian_pts.get(j);
            	sum_x += Math.pow(p.x - g[i].ux, 2);
            	sum_y += Math.pow(p.y - g[i].uy, 2);
        	}
        	g[i].ox = Math.sqrt(sum_x / curr_guassian_pts.size());
        	g[i].oy = Math.sqrt(sum_y / curr_guassian_pts.size());
		}
    }

	/* get_guassian_pts
	 * returns and ArrayList of points that belong to the given guassian 'i'
	 */
    public ArrayList get_guassian_pts(int i)
    {	MyPoint p;
    	ArrayList guassian_pts = new ArrayList();
		for(int j = 0; j < points.size(); j++)
        {	p = (MyPoint)points.get(j);
        	if(p.curr_cluster == i) guassian_pts.add(p);
        }
        return guassian_pts;
    }

	/* converged
	 * returns true if no datapoints switched clusters in the last iteration
	 * of EM
	 */
	public boolean converged()
	{	boolean converged = false;
		int i = 0;
		for(; i < points.size() && ((MyPoint)points.get(i)).converged(); i++);
		return i == points.size();
	}

	/* Guassian
	 * class to represent a 2 dimensional guassian
	 */
	private class Guassian
	{	public double ux, ox, uy, oy;

		public Guassian()
		{	this.ux = -1;
			this.ox = -1;
			this.uy = -1;
			this.oy = -1;
		}

		public Guassian(double ux, double ox, double uy, double oy)
		{	this.ux = ux;
			this.ox = ox;
			this.uy = uy;
			this.oy = oy;
		}

		/* probability
		 * returns the probability that point 'p' belongs to the guassian
		 */
		public double probability(MyPoint p)
		{	return 1 / (ox * Math.sqrt(2*Math.PI))
				 * Math.exp(-.5 * Math.pow((p.x - ux) / ox, 2))
		     	 * 1 / (oy * Math.sqrt(2*Math.PI))
		     	 * Math.exp(-.5 * Math.pow((p.y - uy) / oy, 2));
		}

		public String toString()
		{	return "[ " + ux + " , " + ox + " , " + uy + " , " + oy + " ]";
		}
	}

	private class MyPoint
	{	public double x,y;
		public int prev_cluster;
		public int curr_cluster;

		public MyPoint(double x, double y)
		{	this.x = x;
			this.y = y;
			this.curr_cluster = -1;
			this.prev_cluster = -2;
		}

		/* set_cluster
		 * assigns the point to the given cluster 'i'
		 */
		public void set_cluster(int i)
		{	prev_cluster = curr_cluster;
			curr_cluster = i;
		}

		/* converged
		 * returns true if the point did not change clusters in the past
		 * iteration of EM
		 */
		public boolean converged()
		{	return (prev_cluster == curr_cluster);
		}

		public String toString()
		{	return "[ " + x  + " , " + y + " ]";
		}
	}
}
