//Name: Justin Fiedler
//Project: HW5: MyFileReader
//Class: CS 540

import java.io.*;	//for BufferedReader
import java.util.*; //for StringTokenizer

/* MyFileReader
 * A class used to read text files for input.  Lines must be read one at a time.
 * Once a line has been read the line can be recieved in tokens via nextToken().
 * Tokenizer string of 'delim'.  This reader also recognizes "//" as a
 * comment indicator.  Anything after "//" on a line will be ignored and
 * not counted as a token.
 *
 *<bugs> none
 *
 */

public class MyFileReader
{	private BufferedReader b;
	private StringTokenizer st;
	private String line;
	private String delim;
	private boolean hasMoreTokens;
	private String nextToken;

	/* Constructor
	 * Creates a new MyFileReader for the file 'fileName'
     * and a deliminator string of 'delim'
	 * <param> fileName: the name of the file to read
     *         delim: string of characters to seperate tokens
	 */
	public MyFileReader(String fileName, String delim) throws FileNotFoundException
	{	b = new BufferedReader(new FileReader(fileName));
		this.delim = delim;
		hasMoreTokens = false;
	}

	/* readLine()
	 * Reads the next line in the file.
	 * <return> string: the line read
	 */
	public String readLine() throws IOException
	{	do
		{	line = b.readLine();
			if(line != null && line.length() == 0) line = null;
		}while(line != null && line.startsWith("//"));
		if(line != null)
		{	st = new StringTokenizer(line, delim, false);
			nextToken = st.nextToken();
			hasMoreTokens = true;
		}
		return line;
	}

	/* nextToken()
	 *	Returns the next token in 'line'. If there are no more tokens
	 * then it throws a IOException.
	 * <return> string: next token
	 */
	public String nextToken() throws IOException
	{	String token = null;
		if(hasMoreTokens)
		{	token = nextToken;
			setNextToken();
		}
		else
		{	throw new IOException();
		}
		return token;
	}

    /* setToken()
	 * Checks the line for more tokens and sets next token equal
         * to any tokens found. If there are no more tokens in the line
	 * it will set nextToken to null and hasMoreTokens to false.
	 */
	private void setNextToken()
	{	if(st.hasMoreTokens())
		{	nextToken = st.nextToken();
			if(nextToken.startsWith("//"))
			{	nextToken = null;
				hasMoreTokens = false;
			}
		}
		else
		{	nextToken = null;
			hasMoreTokens = false;
		}
	}

	/* hasMoreTokens()
	 * Determines if the line has more tokens.
	 * <return> boolean: true if more tokens, false if not
	 */
	public boolean hasMoreTokens()
	{	return hasMoreTokens;
	}
}
